#ifndef DISKCACHE_FILE_NAME_HH
#define DISKCACHE_FILE_NAME_HH
   
// System Header Files   
#include <string>   
#include <list>   
   
// General Header Files   
#include <general/types.hh>   
   
   
namespace diskCache
{
   //: Helper class to tokenize frame file name.
   // 
   // This class is designed to extract necessary pieces of information
   // from file name: ifo, type, gps start time, dt of the data.
   //
   class FileName
   {
   public:
   
      //: Constructor
      FileName( const std::string& file_name, const std::string& dir_path );
   
      //: Validators
      const bool shouldIgnore() const;
      const bool isFrame();
      const bool shouldExclude( const std::list< std::string >& exclude_dirs ) const;

      //: String Accessors
      const std::string getIFOType() const;         
   
      //: Numeric Accessors   
      const INT_4U getStartTime() const;  
      const INT_4U getDt() const;         
   
      //: Static accessors
      static const CHAR& getFrameNameSeparator();  

   private:
   
      const bool hasExtension( const std::string& extension, 
                               const size_t& extension_length ) const;
      void parse();
   
   
      //: File name to parse.
      std::string mName;

      //: Directory path file resides in.
      std::string mDir;   
   
      //: Extracted "IFO-Type-" part of the file name
      std::string mIFOType;
   
      //: Extracted start time - GPS time at the beginning of the first 
      //: frame in the file.
      INT_4S mStartTime;   
   
      //: Extracted dt - total time interval covered by the file, in seconds.
      INT_4S mDt;         
   };
}   
   
#endif   
