#include "framecpp/config.h"

#include <memory>

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/STRING.hh"

#include "framecpp/Version3/FrameSpec.hh"

#include "framecpp/Version3/FrSH.hh"

using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::Description;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;

using namespace FrameCPP::Version_3;

FrSH::
FrSH( )
  : Common::FrSH( )
{
}


FrSH::
FrSH( const std::string& Name, INT_2U ClassId, const std::string& Comment )
  : Common::FrSH( )
{
  m_data.name = Name;
  m_data.classId = ClassId;
  m_data.comment = Comment;
}

FrSH::
FrSH( IStream& Stream )
{
  Stream >> m_data.name
	 >> m_data.classId
	 >> m_data.comment
    ;
}

FrSH::
~FrSH( )
{
}

INT_8U FrSH::
Bytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.name.Bytes( ) +
    sizeof( m_data.classId ) +
    m_data.comment.Bytes( );
}

FrSH* FrSH::
Create( IStream& Stream ) const
{
  return new FrSH( Stream );
}

FrSH* FrSH::
Clone( ) const
{
  return new FrSH( *this );
}

const char* const FrSH::
ObjectStructName( ) const
{
  return StructName( );
}

void FrSH::
Write( OStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.classId
	 << m_data.comment;
}

bool FrSH::
operator==( const FrSH& RHS ) const
{
  return ( ( m_data.name == RHS.m_data.name )
	   && ( m_data.classId == RHS.m_data.classId )
	   && ( m_data.comment == RHS.m_data.comment ) );
}

bool FrSH::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

void FrSH::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  while ( InfoClass != TOCInfo::IC_EOQ )
  {
    int data_type = va_arg( vl, int );
    switch( data_type )
    {
    case TOCInfo::DT_STRING_2:
      {
	Common::STRING< INT_2U >*
	  data = va_arg( vl, Common::STRING< INT_2U >* );
	*data = GetName( );
      }
      break;
    case TOCInfo::DT_INT_2U:
      {
	INT_2U*	data = va_arg( vl, INT_2U* );
	*data = GetClass( );
      }
      break;
    default:
      // Stop processing
      InfoClass = TOCInfo::IC_EOQ;
      continue;
    }
    InfoClass = va_arg( vl, int );
  }
  va_end( vl );
}

const std::string& FrSH::
name( ) const
{
  return GetName( );
}

const INT_2U FrSH::
classId( ) const
{
  return GetClass( );
}

std::auto_ptr< FrameSpec::Object > FrSH::
demote( INT_2U Target, IStream* Stream ) const
{
  throw
    Unimplemented( "Object* FrSH::demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrSH::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  throw
    Unimplemented( "Object* FrSH::promote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}
